==> abridge-sdk-1.0.1.tgz: package/src/sdk.js <==
const { default: axios } = require("axios");
const { ethers } = require("ethers");

import { TransferType } from "../constants/index";
import * as contracts from "../abis/index";
import { config } from "../config/nets";

class RouterSDK {
    constructor(signerOrProvider, optionalParams) {
        this.signerOrProvider = signerOrProvider;
        
        this.chainId = this.signerOrProvider.provider.network.chainId;
        this.config = config[this.chainId];

        this.setParams(optionalParams || {});

        this.routerContract = new ethers.Contract(this.routerAddress, this.routerABI, this.signerOrProvider);
        this.axios = new axios.create({ baseURL: this.validatorURL });
    }

    /**
     * @param optionalParams
     * @property routerABI: RouterContractABI 
     * @property erc20ABI: ERC20Permit 
     * @property routerAddress: address 
     * @property validators: array
     * @property threshold: number 
     */
    setParams(optionalParams) {
        
        if (optionalParams["routerABI"]) {
            this.routerABI = optionalParams.routerABI;
        } else {
            this.routerABI = contracts.routerContract.abi;
        }
        
        if (optionalParams["erc20ABI"]) {
            this.erc20ABI = optionalParams.erc20ABI;
        } else {
            this.erc20ABI = contracts.ERC20Permit.abi;
        }
        
        if (optionalParams["routerAddress"]) {
            this.routerAddress = optionalParams.routerAddress;
        } else {
            this.routerAddress = this.config.contracts.router;
        }
        
        if (optionalParams["validators"]) {
            this.validatorURL = this.getRandomProperty(optionalParams.validators);
        } else {
            this.validatorURL = this.getRandomProperty(this.config.validators);
        }

        let threshold;
        if (optionalParams["threshold"]) {
            threshold = ethers.utils.formatUnits(optionalParams.threshold)
        } else {
            threshold = ethers.utils.formatUnits(this.config.threshold)
        }

        this.tokenAllowanceThreshold = ethers.utils.parseUnits(threshold);
    }
    
    getRandomProperty(obj) {
        const keys = Object.keys(obj);
        const randomIndex = keys.length * Math.random() << 0;

        return obj[keys[randomIndex]];
    }

    async send(targetChainId, tokenAddress, amount, receiver) {
        const nativeToken = await this.routerContract.wrappedToNativeToken(tokenAddress);
        if(nativeToken.chainId == 0) {
            return await this.routerContract.populateTransaction.lock(targetChainId, tokenAddress, amount, receiver);
        }

        return await this.routerContract.populateTransaction.burn(tokenAddress, amount, receiver);
    }

    async sendWithPermit(targetChainId, tokenAddress, amount, receiver, v, r, s, deadline) {
        const nativeToken = await this.routerContract.wrappedToNativeToken(tokenAddress);
        if(nativeToken.chainId == 0) {
            return await this.routerContract.populateTransaction.lockWithPermit(targetChainId, tokenAddress, amount, receiver, deadline, v, r, s);
        }

        return await this.routerContract.populateTransaction.burnWithPermit(tokenAddress, amount, receiver, deadline, v, r, s);
    }

    async claim(transfer) {
        switch(transfer.transferType) {
            case  TransferType.LOCK:
                return await this.routerContract.populateTransaction.mint(
                    transfer.sourceChainId,
                    transfer.nativeToken,
                    ethers.utils.toUtf8Bytes(transfer.sourceTx),
                    transfer.amount,
                    transfer.recipient,
                    this.fixSignatures(transfer.signatures),
                    { name: transfer.wrappedTokenName,
                    symbol: transfer.wrappedTokenSymbol,
                    decimals: transfer.wrappedTokenDecimals }
                );
            case  TransferType.BURN:
                return await this.routerContract.populateTransaction.unlock(
                    transfer.sourceChainId,
                    ethers.utils.toUtf8Bytes(transfer.sourceTx),
                    transfer.nativeToken,
                    transfer.amount,
                    transfer.recipient,
                    this.fixSignatures(transfer.signatures)
                );
        }

        return null;
    }

    async balanceOf(tokenAddress, userAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        const balance = await tokenContract.balanceOf(userAddress);
        const tokenDecimals = await tokenContract.decimals();
        return { balance, tokenDecimals };
    }

    async allowance(tokenAddress, userAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        const tokenDecimals = await tokenContract.decimals();
        const allowance = await tokenContract.allowance(userAddress, this.routerContract.address);
        return { allowance, tokenDecimals };
    }

    async isTokenApproved(tokenAddress, userAddress, threshold /** optional */) {
        const allowanceThreshold = threshold || this.tokenAllowanceThreshold;
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        const currentAllowance = await tokenContract.allowance(userAddress, this.routerContract.address);
        return currentAllowance.gte(allowanceThreshold);
    }

    async approve(tokenAddress, amount) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return await tokenContract.populateTransaction.approve(this.routerContract.address, amount);
    }

    async approveMaxInt(tokenAddress) {
        return this.approve(tokenAddress, ethers.constants.MaxUint256);
    }

    async getServiceFee() {
        return await this.routerContract.serviceFee();
    }

    async getClaims(userAddress, targetChainId) {
        const transfers = await this.axios(`/getTransfers?targetChainId=${targetChainId}&recipient=${userAddress}`, {responseType: "json"});
        return transfers.data.transfers;
    }

    async getWrappedTokens(chainId) {
        const wrappedTokens = await this.axios(`/getWrappedTokens?chainId=${chainId}`, {responseType: "json"});
        return wrappedTokens.data.wrappedTokens;
    }

    async isClaimed(transfer) {
        let encodedData;
        switch(transfer.transferType) {
            case  TransferType.BURN:
                encodedData = ethers.utils.defaultAbiCoder.encode(
                    [
                        "uint8",
                        "uint8",
                        "bytes",
                        "address",
                        "uint256",
                        "bytes"
                    ],
                    [
                        transfer.sourceChainId,
                        transfer.targetChainId,
                        ethers.utils.toUtf8Bytes(transfer.sourceTx),
                        transfer.recipient,
                        transfer.amount,
                        transfer.nativeToken
                    ]
                );
                break;
            case  TransferType.LOCK:
                encodedData = ethers.utils.defaultAbiCoder.encode(
                    [
                        "uint8",
                        "uint8",
                        "bytes",
                        "address",
                        "uint256",
                        "bytes",
                        "string",
                        "string",
                        "uint8"
                    ],
                    [
                        transfer.sourceChainId,
                        transfer.targetChainId,
                        ethers.utils.toUtf8Bytes(transfer.sourceTx),
                        transfer.recipient,
                        transfer.amount,
                        transfer.nativeToken,
                        transfer.wrappedTokenName,
                        transfer.wrappedTokenSymbol,
                        transfer.wrappedTokenDecimals
                    ]
                );
                break;
        }
        
        const hashMsg = ethers.utils.keccak256(encodedData);
        const hashData = ethers.utils.arrayify(hashMsg);

        return await this.routerContract.hashesUsed(transfer.sourceChainId, ethers.utils.hashMessage(hashData));
    }

    async getTokenName(tokenAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return await tokenContract.name();
    }

    async getTokenSymbol(tokenAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return await tokenContract.symbol();
    }

    async getTokenDecimals(tokenAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return await tokenContract.decimals();
    }

    async getTokenDetails(tokenAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return {
            name: await tokenContract.name(),
            symbol: await tokenContract.symbol(),
            decimals: await tokenContract.decimals()
        }
    }

    async getWrappedTokenDetails(nativeTokenAddress, nativeChainId) {
        const wrappedTokenAddress = await this.routerContract.nativeToWrappedToken(nativeChainId, nativeTokenAddress);
        const wrappedTokenDetails = await this.getTokenDetails(wrappedTokenAddress);
        return { address: wrappedTokenAddress, ...wrappedTokenDetails };
    }

    async getNativeTokenByWrappedAddress(wrappedTokenAddress) {
        return await this.routerContract.wrappedToNativeToken(wrappedTokenAddress);
    }

    async getALBTToken() {
        const albtAddress = await this.routerContract.albtToken();
        const albtDetails = await this.getTokenDetails(albtAddress);
        return { address: albtAddress, ...albtDetails };
    }

    fixSignatures(signatures) {
        const fixedSignatures = [];
        for(let i = 0; i < signatures.length; i++) {
            fixedSignatures[i] = signatures[i].startsWith("0x") ? signatures[i] : "0x" + signatures[i];
        }
    
        return fixedSignatures;
    }
}

export { RouterSDK };

==> abridge-sdk-1.0.2.tgz: package/src/sdk.js <==
const { default: axios } = require("axios");
const { ethers } = require("ethers");

import { TransferType } from "../constants/index";
import * as contracts from "../abis/index";
import { config } from "../config/nets";

class RouterSDK {
    constructor(signerOrProvider, optionalParams) {
        this.signerOrProvider = signerOrProvider;
        
        this.chainId = this.signerOrProvider.provider.network.chainId;
        this.config = config[this.chainId];

        this.setParams(optionalParams || {});

        this.routerContract = new ethers.Contract(this.routerAddress, this.routerABI, this.signerOrProvider);
        this.axios = new axios.create({ baseURL: this.validatorURL });
    }

    /**
     * @param optionalParams
     * @property routerABI: RouterContractABI 
     * @property erc20ABI: ERC20Permit 
     * @property routerAddress: address 
     * @property validators: array
     * @property threshold: number 
     */
    setParams(optionalParams) {
        
        if (optionalParams["routerABI"]) {
            this.routerABI = optionalParams.routerABI;
        } else {
            this.routerABI = contracts.routerContract.abi;
        }
        
        if (optionalParams["erc20ABI"]) {
            this.erc20ABI = optionalParams.erc20ABI;
        } else {
            this.erc20ABI = contracts.ERC20Permit.abi;
        }
        
        if (optionalParams["routerAddress"]) {
            this.routerAddress = optionalParams.routerAddress;
        } else {
            this.routerAddress = this.config.contracts.router;
        }
        
        if (optionalParams["validators"]) {
            this.validatorURL = this.getRandomProperty(optionalParams.validators);
        } else {
            this.validatorURL = this.getRandomProperty(this.config.validators);
        }

        let threshold;
        if (optionalParams["threshold"]) {
            threshold = ethers.utils.formatUnits(optionalParams.threshold)
        } else {
            threshold = ethers.utils.formatUnits(this.config.threshold)
        }

        this.tokenAllowanceThreshold = ethers.utils.parseUnits(threshold);
    }
    
    getRandomProperty(obj) {
        const keys = Object.keys(obj);
        const randomIndex = keys.length * Math.random() << 0;

        return obj[keys[randomIndex]];
    }

    async send(targetChainId, tokenAddress, amount, receiver) {
        const nativeToken = await this.routerContract.wrappedToNativeToken(tokenAddress);
        if(nativeToken.chainId == 0) {
            return await this.routerContract.populateTransaction.lock(targetChainId, tokenAddress, amount, receiver);
        } else if (nativeToken.chainId !== targetChainId) {
            return await this.routerContract.populateTransaction.burnAndTransfer(targetChainId, tokenAddress, amount, receiver);
        }

        return await this.routerContract.populateTransaction.burn(tokenAddress, amount, receiver);
    }

    async sendWithPermit(targetChainId, tokenAddress, amount, receiver, v, r, s, deadline) {
        const nativeToken = await this.routerContract.wrappedToNativeToken(tokenAddress);
        if(nativeToken.chainId == 0) {
            return await this.routerContract.populateTransaction.lockWithPermit(targetChainId, tokenAddress, amount, receiver, deadline, v, r, s);
        } else if (nativeToken.chainId !== targetChainId) {
            return await this.routerContract.populateTransaction.burnAndTransferWithPermit(targetChainId, tokenAddress, amount, receiver, deadline, v, r, s);
        }

        return await this.routerContract.populateTransaction.burnWithPermit(tokenAddress, amount, receiver, deadline, v, r, s);
    }

    async claim(transfer) {
        switch(transfer.transferType) {
            case TransferType.LOCK:
                return await this.routerContract.populateTransaction.mint(
                    transfer.nativeChainId === 0 ? transfer.sourceChainId : transfer.nativeChainId,
                    transfer.nativeToken,
                    ethers.utils.toUtf8Bytes(transfer.sourceTx),
                    transfer.amount,
                    transfer.recipient,
                    this.fixSignatures(transfer.signatures),
                    { name: transfer.wrappedTokenName,
                    symbol: transfer.wrappedTokenSymbol,
                    decimals: transfer.wrappedTokenDecimals }
                );
            case TransferType.BURN:
                return await this.routerContract.populateTransaction.unlock(
                    transfer.sourceChainId,
                    ethers.utils.toUtf8Bytes(transfer.sourceTx),
                    transfer.nativeToken,
                    transfer.amount,
                    transfer.recipient,
                    this.fixSignatures(transfer.signatures)
                );
            case TransferType.BURN_AND_TRANSFER:
                return await this.routerContract.populateTransaction.mint(
                    transfer.nativeChainId,
                    transfer.nativeToken,
                    ethers.utils.toUtf8Bytes(transfer.sourceTx),
                    transfer.amount,
                    transfer.recipient,
                    this.fixSignatures(transfer.signatures),
                    { name: transfer.wrappedTokenName,
                    symbol: transfer.wrappedTokenSymbol,
                    decimals: transfer.wrappedTokenDecimals }
                );
        }

        return null;
    }

    async balanceOf(tokenAddress, userAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        const balance = await tokenContract.balanceOf(userAddress);
        const tokenDecimals = await tokenContract.decimals();
        return { balance, tokenDecimals };
    }

    async allowance(tokenAddress, userAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        const tokenDecimals = await tokenContract.decimals();
        const allowance = await tokenContract.allowance(userAddress, this.routerContract.address);
        return { allowance, tokenDecimals };
    }

    async isTokenApproved(tokenAddress, userAddress, threshold /** optional */) {
        const allowanceThreshold = threshold || this.tokenAllowanceThreshold;
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        const currentAllowance = await tokenContract.allowance(userAddress, this.routerContract.address);
        return currentAllowance.gte(allowanceThreshold);
    }

    async approve(tokenAddress, amount) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return await tokenContract.populateTransaction.approve(this.routerContract.address, amount);
    }

    async approveMaxInt(tokenAddress) {
        return this.approve(tokenAddress, ethers.constants.MaxUint256);
    }

    async getServiceFee() {
        return await this.routerContract.serviceFee();
    }

    async getClaims(userAddress, targetChainId, page = 'undefined') {
        const transfers = await this.axios(`/getTransfers?targetChainId=${targetChainId}&recipient=${userAddress}&page=${page}`, {responseType: "json"});
        return { claims: transfers.data.transfers, totalCount: transfers.headers['x-total-count']}
    }

    async getWrappedTokens(chainId) {
        const wrappedTokens = await this.axios(`/getWrappedTokens?chainId=${chainId}`, {responseType: "json"});
        return wrappedTokens.data.wrappedTokens;
    }

    async isClaimed(transfer) {
        let encodedData;
        switch(transfer.transferType) {
            case TransferType.BURN:
                encodedData = ethers.utils.defaultAbiCoder.encode(
                    [
                        "uint8",
                        "uint8",
                        "bytes",
                        "address",
                        "uint256",
                        "bytes"
                    ],
                    [
                        transfer.sourceChainId,
                        transfer.targetChainId,
                        ethers.utils.toUtf8Bytes(transfer.sourceTx),
                        transfer.recipient,
                        transfer.amount,
                        transfer.nativeToken
                    ]
                );
                break;
            case TransferType.LOCK:
                // Backwards compatibility fix
                if (transfer.nativeChainId === 0) {
                    transfer.nativeChainId = transfer.sourceChainId;
                }
                encodedData = ethers.utils.defaultAbiCoder.encode(
                    [
                        "uint8",
                        "uint8",
                        "bytes",
                        "address",
                        "uint256",
                        "bytes",
                        "string",
                        "string",
                        "uint8"
                    ],
                    [
                        transfer.nativeChainId,
                        transfer.targetChainId,
                        ethers.utils.toUtf8Bytes(transfer.sourceTx),
                        transfer.recipient,
                        transfer.amount,
                        transfer.nativeToken,
                        transfer.wrappedTokenName,
                        transfer.wrappedTokenSymbol,
                        transfer.wrappedTokenDecimals
                    ]
                );
                break;
            case TransferType.BURN_AND_TRANSFER:
                encodedData = ethers.utils.defaultAbiCoder.encode(
                    [
                        "uint8",
                        "uint8",
                        "bytes",
                        "address",
                        "uint256",
                        "bytes",
                        "string",
                        "string",
                        "uint8"
                    ],
                    [
                        transfer.nativeChainId,
                        transfer.targetChainId,
                        ethers.utils.toUtf8Bytes(transfer.sourceTx),
                        transfer.recipient,
                        transfer.amount,
                        transfer.nativeToken,
                        transfer.wrappedTokenName,
                        transfer.wrappedTokenSymbol,
                        transfer.wrappedTokenDecimals
                    ]
                );
                break;
        }
        
        const hashMsg = ethers.utils.keccak256(encodedData);
        const hashData = ethers.utils.arrayify(hashMsg);

        return await this.routerContract.hashesUsed(
                transfer.transferType == TransferType.BURN ? transfer.sourceChainId : transfer.nativeChainId, 
                ethers.utils.hashMessage(hashData)
            );
    }

    async getTokenName(tokenAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return await tokenContract.name();
    }

    async getTokenSymbol(tokenAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return await tokenContract.symbol();
    }

    async getTokenDecimals(tokenAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return await tokenContract.decimals();
    }

    async getTokenDetails(tokenAddress) {
        const tokenContract = new ethers.Contract(tokenAddress, this.erc20ABI, this.signerOrProvider);
        return {
            name: await tokenContract.name(),
            symbol: await tokenContract.symbol(),
            decimals: await tokenContract.decimals()
        }
    }

    async getWrappedTokenDetails(nativeTokenAddress, nativeChainId) {
        const wrappedTokenAddress = await this.routerContract.nativeToWrappedToken(nativeChainId, nativeTokenAddress);
        const wrappedTokenDetails = await this.getTokenDetails(wrappedTokenAddress);
        return { address: wrappedTokenAddress, ...wrappedTokenDetails };
    }

    async getNativeTokenByWrappedAddress(wrappedTokenAddress) {
        return await this.routerContract.wrappedToNativeToken(wrappedTokenAddress);
    }

    async getALBTToken() {
        const albtAddress = await this.routerContract.albtToken();
        const albtDetails = await this.getTokenDetails(albtAddress);
        return { address: albtAddress, ...albtDetails };
    }

    fixSignatures(signatures) {
        const fixedSignatures = [];
        for(let i = 0; i < signatures.length; i++) {
            fixedSignatures[i] = signatures[i].startsWith("0x") ? signatures[i] : "0x" + signatures[i];
        }
    
        return fixedSignatures;
    }
}

export { RouterSDK };
